import { AgoraAppStore } from "any-hooks/types/agora";
import { onMounted, watch } from "vue";
import { useInjector, useState } from "vue-vulcan";
import { AGORA_APP_SDK } from "../communication/token";

interface AgoraStreamItem {
  url?: string;
  uid: string;
}

// type AgoraState = 'unready' | 'inited' | 'joined';

/** 基于声网sdk封装，由于多平台特性，请在根组件将对应平台的声网SDK对象以及appid通过useProvider提供; */
export function useAgoraClient() {  
  const [agoraErr, setError]            = useState(null);
  const [streamList, setStream]         = useState<AgoraStreamItem[]>([]);
  const { appid, client }               = useInjector<AgoraAppStore>(AGORA_APP_SDK);

  onMounted(() => {
    client.init(appid, subscribeRemoteStream, setError);
  })

  const joinChannelWithVideo = (token: string, cid: string, uid: string) => {
    client.join(
      token, 
      cid, 
      uid, 
      pushVideoStream, 
      setError
    );
  }

  //推送本地视频流
  const [localPushurl, setPushUrl]  = useState('');
  const pushVideoStream = () => {
    client.publish( url => setPushUrl(url));
  }

  //监听远程视频流
  const subscribeRemoteStream = () => {
    client.on('stream-added', evt => {
      client.subscribe(
        evt.uid, 
        url => updateStremList('add', {uid: evt.uid, url}),
        setError
      )
    });
    client.on('stream-removed', evt => updateStremList('remove', {uid: evt.uid}));
  }
  watch(streamList, console.log)

  // 根据uid来新增或移除远端视频流列表
  const updateStremList = (action: 'add' | 'remove', input: AgoraStreamItem) => {
    switch(action) {
      case 'add':
        setStream(streamList.value.concat(input));
      break;
      case 'remove':
        const index = streamList.value.findIndex(item => item.uid === input.uid);
        index && setStream(streamList.value.splice(index, 1));
      break;
    }
  }

  // 监听来自Agora Client的错误提示
  watch(agoraErr, err => {
    console.error(err)
  })


  return {
    localPushurl,
    streamList,
    joinChannelWithVideo
  }
}