import { AnyRemoteSubFlag } from "any-hooks/types/socket";
import { UserData } from "any-hooks/types/user";
import { watch } from "vue";
import { useInjector, useState } from "vue-vulcan";
import { useChannelStore } from "./useChannelStore";
import { useNetSocketCenter } from "./useNetSocketCenter";

type CallingState = 'net_error' | 'calling' | 'call_successed' | 'call_accepted' | 'free';

interface Caller extends UserData {  
  action: AnyRemoteSubFlag | 'none';
  channel?: string;
}

/**  呼叫中心功能，可以主动呼叫联系人、回应对方的呼叫，以及监听双方呼叫人的信息和状态 */
export function useCallCenter() {  

  const { sendMsg, currentMsg } = useInjector(useNetSocketCenter);
  
  /** 主动呼叫功能 */
  const [target, setTarget]             = useState<Caller>(null);
  const [myCallState, setCallState]     = useState<CallingState>('free');  
  const {currentChannel, createChannel, joinCallingChannel} = useInjector(useChannelStore);
  const callContact = (user: UserData) => {
    setTarget({...user, action: 'none'});      //保存呼叫目标的信息
    setCallState('calling'); //主动呼叫别人时将自己的呼叫状态更改为‘calling’
    createChannel().then( _ => {
      sendMsg({
        toID: user.id,
        toName: user.nickname,
        channelID: currentChannel.value.channel_id,
        msgMainFlag: 'CallOffer',
        msgSubFlag: 'Request'
      })
    })    
  }

  /** 回应呼叫功能 */
  const answerCaller = (subFlag: AnyRemoteSubFlag) => {
    if(subFlag === 'Connect') {
      setCallState('call_accepted');
      joinCallingChannel(caller.value.channel);
    };
    sendMsg({      
      toID: caller.value.id,
      toName: caller.value.nickname,
      msgMainFlag: 'CallAnswer',
      msgSubFlag: subFlag
    })
  }

  /** 呼叫监听功能  */
  const [caller, setCaller] = useState<Caller>(null);  
  // 监听来自其他用户的呼叫请求
  watch(currentMsg, msg => {
    if(msg.msgMainFlag !== 'CallOffer') return;
    if(myCallState.value !== 'free') answerCaller('Busying');
    console.log(msg.msgSubFlag);
    setCaller({      
      id: currentMsg.value.fromID,
      nickname: currentMsg.value.fromName,  
      action: msg.msgSubFlag,
      channel: msg.channelID
    })    
  })
  // 监听呼叫目标的回应
  watch(currentMsg, msg => {
    if(msg.msgMainFlag !== 'CallAnswer') return;
    switch(msg.msgSubFlag) {
      case 'Connect':
        setCallState('call_successed');
      break;
      case 'Busying':
      case 'Hangup':
        console.log('对方拒绝呼叫请求')
        setCallState('free');
        setTarget({...target.value, action: msg.msgSubFlag});       
      break; 
    }
  })  

  /** 呼叫挂断功能 */
  const hangup = () => {    
    sendMsg({      
      channelID: currentChannel.value.channel_id,
      msgMainFlag: myCallState.value === 'calling' ? 'CallOffer' : 'CallAnswer',
      msgSubFlag: 'Hangup',
      toID: myCallState.value === 'calling' ? target.value.id : caller.value.id
    })
    setCallState('free');
  }

  return {
    /** 呼叫远程联系人 @param UserData */
    callContact,    
    /** 回应远程联系人的呼叫 @param AnyRemoteSubFlag  */
    answerCaller,
    /** 挂断当前呼叫  */
    hangup,
    /** 被本地用户呼叫目标联系人  */
    target,
    /** 向本地用户发起呼叫的远程联系人 */
    caller,
    /** 本地用户的呼叫状态 */
    myCallState,
  }
}