import { watch } from "vue";
import { useRequest, useState } from "vue-vulcan";
import { UserData } from "any-hooks/types/user";

interface ChannelData {
  channel_id: string;
  agora_token: string;
  members?: UserData[];
}

/** 频道中心功能，可以创建频道，并保存当前频道的信息，供呼叫中心、会议中心使用（目前只允许同一时间仅存在一个频道） */
export function useChannelStore() {
  const [currentChannel, setCurrent]   = useState<ChannelData>(null);
  const [channelInfo, request]         = useRequest<ChannelData>('/getAgoraToken', {auto: false}); 

  const createChannel = () => {
    return request()
  }  

  const joinCallingChannel = (cid: string) => {
    request({channel_id: cid});
  }  

  watch(channelInfo, val => {    
    setCurrent(val);    
  })


  return {
    currentChannel,
    joinCallingChannel,
    createChannel    
  }
}