import { useAuthData } from "any-hooks/auth/useAuthData";
import { useSocket } from "any-hooks/common/useSocket";
import { AnyRemoteSocketMessage } from "any-hooks/types/socket";
import { watch } from "vue";
import { useInjector, useState } from "vue-vulcan";

enum NetStates {
  off = 0,
  bad,
  normal,
  good,
  best
}

export function useNetSocketCenter() {
  
  const { authData }                  = useInjector(useAuthData);   
  const [ netState, setNetState ]     = useState<NetStates>(0);
  const { connect, send, currentMsg, startHeartConnect } = useSocket<AnyRemoteSocketMessage>();

  watch(authData, data => {
    if(!data) return;
    connect(
      `wss://www.if-ar.com:3009?fromID=${data.id}&fromName=${data.nickname}&signID=SA&companyID=${data.company_id}`
    )
    startHeartConnect({
      fromID: authData.value.id,
      fromName: authData.value.nickname,
      toID: '0',
      msgMainFlag: 'Heart'
    })
  })

  const sendMsg = (data: AnyRemoteSocketMessage) => {
    send({
      ...data, 
      fromID: authData.value.id, 
      fromName: authData.value.nickname, 
      avatar: authData.value.avatar
    })
  }

  return {
    sendMsg,
    setNetState,
    currentMsg,
    netState
  }
}