import { navigateBack, navigateTo, redirectTo, showModal, showToast } from "@tarojs/taro";
import { useCallCenter } from "any-hooks/communication/useCallCenter";
import { watch } from "vue";
import { useInjector } from "vue-vulcan";


/** 在小程序平台监听远程联系人的呼叫动作 */
export function useCallerListener() {
  const { caller, myCallState, answerCaller } = useInjector(useCallCenter);

  watch(caller, current => {        
    switch(current.action) {
      case 'Request':
        showModal({
          content: `来自${current.nickname}的呼叫，是否接受?`,
          cancelText: '拒绝',
          cancelColor: 'red',
          confirmText: '接受',
          success: res => {              
            if(caller.value.action !== 'Request') return; //此次必须通过caller访问action值
            res.confirm ? answerCaller('Connect') : answerCaller('Hangup');          
          },        
          fail: () => answerCaller('Hangup')
        })
      break;
      case 'Hangup':          
        showToast({title: '对方取消了呼叫,您可以点击接受关闭对话框', icon: 'none'});
      break;
    }  
  })

  // 根据用户的呼叫状态变化，执行相应的页面跳转逻辑
  watch(myCallState, state => {
    switch(state) {
      case 'call_accepted':
        navigateTo({url: '/pages/meeting/index'});
      break;
      case 'call_successed':    
        redirectTo({url: '/pages/meeting/index'});
      break;
      case 'calling':
        navigateTo({url: '/pages/calling/index'});
      break;
      case 'free':
        navigateBack({delta: 2})
      break;
    }
  })
}